PYTHON_3_8_DOCKERFILE_TEMPLATE = """FROM python:3.8
COPY ./ /tmp/
RUN pip install --upgrade pip
RUN pip install -r /tmp/requirements.txt
RUN useradd -u 9097 code_runner
USER code_runner:code_runner
WORKDIR /tmp
CMD ["bash", "-c", "python main.py > /tmp/logs/execution.log 2>&1"]
"""

PYTHON_3_9_DOCKERFILE_TEMPLATE = """FROM python:3.9
COPY ./ /tmp/
RUN pip install --upgrade pip
RUN pip install -r /tmp/requirements.txt
RUN useradd -u 9097 code_runner
USER code_runner:code_runner
WORKDIR /tmp
CMD ["bash", "-c", "python main.py > /tmp/logs/execution.log 2>&1"]
"""

PYTHON_3_10_DOCKERFILE_TEMPLATE = """FROM python:3.10
COPY ./ /tmp/
RUN pip install --upgrade pip
RUN pip install -r /tmp/requirements.txt
RUN useradd -u 9097 code_runner
USER code_runner:code_runner
WORKDIR /tmp
CMD ["bash", "-c", "python main.py > /tmp/logs/execution.log 2>&1"]
"""

PYTHON_3_11_DOCKERFILE_TEMPLATE = """FROM python:3.11
COPY ./ /tmp/
RUN pip install --upgrade pip
RUN pip install -r /tmp/requirements.txt
RUN useradd -u 9097 code_runner
USER code_runner:code_runner
WORKDIR /tmp
CMD ["bash", "-c", "python main.py > /tmp/logs/execution.log 2>&1"]
"""

PYTHON_3_12_DOCKERFILE_TEMPLATE = """FROM python:3.12
COPY ./ /tmp/
RUN pip install --upgrade pip
RUN pip install -r /tmp/requirements.txt
RUN useradd -u 9097 code_runner
USER code_runner:code_runner
WORKDIR /tmp
CMD ["bash", "-c", "python main.py > /tmp/logs/execution.log 2>&1"]
"""

NODE_24_DOCKERFILE_TEMPLATE = """FROM node:24
COPY ./ /tmp/
WORKDIR /tmp
RUN npm install --production --loglevel verbose
RUN useradd -u 9097 code_runner
USER code_runner:code_runner
CMD ["bash", "-c", "node main.js > /tmp/logs/execution.log 2>&1"]
"""

R_4_5_DOCKERFILE_TEMPLATE = """FROM r-base:4.5.0
COPY ./ /tmp/
WORKDIR /tmp
RUN apt-get update && apt-get install -y \
    libgmp-dev libcurl4-openssl-dev libssl-dev libxml2-dev \
    && rm -rf /var/lib/apt/lists/*
RUN Rscript -e "install.packages('renv', repos='https://cran.rstudio.com/')"
RUN mkdir -p /tmp/renv/.cache
ENV RENV_PATHS_CACHE=/tmp/renv/.cache
RUN Rscript -e "tryCatch(renv::restore(prompt = FALSE), error = function(e) { message('Package installation failed: ', e); quit(status = 1) })"
RUN useradd -u 9097 -m code_runner
ENV HOME=/home/code_runner
USER code_runner:code_runner
CMD ["bash", "-c", "Rscript main.R > /tmp/logs/execution.log 2>&1"]
"""


import os
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(os.path.dirname(__file__)), "raise_sdk")))
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(os.path.dirname(__file__)), "raise_sdk", "revo")))
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(os.path.dirname(__file__)), "raise_sdk", "revo", "app")))

from pathlib import Path
import pytest
from raise_sdk.revo.LocalCodeRunner import LocalCodeRunner
from raise_sdk.utils.file_utils import find_local_paths

# ===============================================================
# --- Fixtures for setting up dummy files and runner instance ---
# ===============================================================
# In these tests we use pytest's temporary directory fixture (tmp_path) to avoid
# polluting your real filesystem and we "dummy‐patch" external dependencies (such
# as the docker connection and file lookup) so that our tests are self‑contained.

@pytest.fixture
def dummy_files(tmp_path):
    """
    Create dummy script, requirements, and dataset files in a temporary directory.
    Returns three lists (each containing one file path as a string).
    """
    script_file = tmp_path / "main.py"
    script_file.write_text("print('Hello from script')")
    
    req_file = tmp_path / "requirements.txt"
    req_file.write_text("package1==1.0")
    
    dataset_file = tmp_path / "datafile.csv"
    dataset_file.write_text("id,value\n1,100")
    
    return [str(script_file)], [str(req_file)], [str(dataset_file)]

@pytest.fixture
def local_runner(tmp_path, dummy_files, monkeypatch):
    """
    Instantiate a LocalCodeRunner with dummy file paths.
    Patch os.getcwd so that the experiment directory is created in tmp_path.
    Also, patch external dependencies like connect_with_docker and find_local_paths.
    """
    # Ensure that os.getcwd() returns our temporary directory.
    monkeypatch.setattr(os, "getcwd", lambda: str(tmp_path))
    
    """# Patch connect_with_docker to always return None (simulate no Docker available)
    monkeypatch.setattr(
        LocalCodeRunner,
        "connect_with_docker",
        lambda logger: None
    )"""

    # Patch find_local_paths to return a mapping from each file's Path to its basename
    def dummy_find_local_paths(paths):
        return {Path(p): os.path.basename(p) for p in paths}
    
    monkeypatch.setattr(
        "raise_sdk.utils.file_utils.find_local_paths",  # Correct patching path
        dummy_find_local_paths  # The mock function
    )
    
    script_paths, req_paths, dataset_paths = dummy_files
    # Instantiate LocalCodeRunner with our dummy file paths.
    runner = LocalCodeRunner(
        script               = script_paths,
        requirements         = req_paths,
        dataset              = dataset_paths,
        programming_language = "python",
        version              = "3.11",
    )
    return runner



# -*- coding: utf-8 -*-
"""
    RAISE - RAI Synthetic Data Generator

    @author: Mikel Hernández Jiménez - Vicomtech Foundation, Basque Research and Technology Alliance (BRTA)
    @version: 0.1
"""
# Stdlib imports
import os
import time

# Third-party app imports
from celery import Celery
from celery.utils.log import get_task_logger
import requests
import docker

# import docker

# Imports from your apps
from code_runner_files.Config import CeleryConfigProcessingScripts
from CodeRunner import CodeRunner

# from code_runner_files.LogClass import LogClass
from code_runner_files.custom_exceptions import MinioConnectionError, DockerConnectionError

celery_app = Celery()
celery_app.config_from_object(CeleryConfigProcessingScripts)

logger = get_task_logger(__name__)

TIME_OUT_ERROR = 2
SYNTAX_ERROR = 1


@celery_app.task(name=os.environ.get("PROCESSING_SCRIPTS_TOPIC", "processing-scripts"), max_retries=2)
def processing_script_execution(experiment_id,dataset_id_list):

    logger.info("⌛ Processing script execution task received with experiment_id = " + experiment_id)
    registration_url = os.environ.get("RCN_EXPERIMENT_REGISTRATION_ENDPOINT", "http://rcn-api:8000/register_processing_script_results/request")

    try:
        # Create the CodeRunner class
        code_runner = CodeRunner(experiment_id=experiment_id, dataset_id_list=dataset_id_list, logger=logger)
        logger.info("- Experiment path: " + code_runner.experiment_path)
        logger.info("- Root path: " + code_runner.root_path)
        logger.info("- Experiment id: " + code_runner.experiment_id)

        # Prepare the experiment path that contains the files needed for running the code
        code_runner.prepare_experiment_path()
        logger.info("🥳 Experiment path prepared for running the code.")
        
        try:
            code_runner.prepare_data_from_minio()
            logger.info("🥳 Data retrieved from minio correctly")
        except MinioConnectionError:
            logger.exception("There has been an error retrieving the data from minio")

        try:
            # Build docker image for experiment
            logger.info("⌛ Building docker image for experiment")
            code_runner.build_docker_image()
            logger.info(
                f"🥳 Docker image generated for experiment_id = {experiment_id}. ⌛ Running the code for the"
                " experiment..."
            )
        except docker.errors.BuildError as exception:
            logger.error(f"Cannot build image img_raise_{experiment_id}. BuildError: {exception}")
            # Notify the RCN that the experiment execution has not been done
            params = {"experiment_id": experiment_id, "status_code": TIME_OUT_ERROR}
            try:
                rcn_response = requests.post(registration_url, params=params)
                logger.info(
                    f"""RCN API experiment registration request:{params},
                        status_code {rcn_response.status_code},
                        response_request: {rcn_response.text}"""
                )
                return False
            except Exception:
                logger.exception("Cannot connect to RCN API")
            for line in exception.build_log:
                if 'error' in line:
                    logger.error("Build log error:", line['error'])
                elif 'stream' in line:
                    logger.info(line['stream'].strip())
            return False

        # Execute the processing script
        start_time = time.time()
        logger.info("⌛ Running docker container for experiment")
        code_runner.run_docker_container()
        if code_runner.status_code == TIME_OUT_ERROR:
            logger.error(f"Cannot execute experiment because of Timeout error")
            # Notify the RCN that the experiment execution has not been done
            params = {"experiment_id": experiment_id, "status_code": TIME_OUT_ERROR}
            try:
                rcn_response = requests.post(registration_url, params=params)
                logger.info(
                    f"""RCN API experiment registration request:{params},
                        status_code {rcn_response.status_code},
                        response_request: {rcn_response.text}"""
                )
                return False
            except Exception:
                logger.exception("Cannot connect to RCN API")
        else:
            logger.info(
                f"🥳 Code successfully runned for experiment_id = {experiment_id}. ⌛ Checking the results of the"
                " experiment..."
            )
            logger.info(f"Elapsed time: {time.time()-start_time}")

        # upload data to minio
        try:
            code_runner.upload_results_to_minio_and_execution_logs()
        except Exception:
            logger.exception("Could not upload data to minio")
            #TODO: make a real exception
            logger.error(f"Experiment results storage failed")
            # Notify the RCN that the experiment execution has not been done
            params = {"experiment_id": experiment_id, "status_code": TIME_OUT_ERROR}
            try:
                rcn_response = requests.post(registration_url, params=params)
                logger.info(
                    f"""RCN API experiment registration request:{params},
                        status_code {rcn_response.status_code},
                        response_request: {rcn_response.text}"""
                )
                return False
            except Exception:
                logger.exception("Cannot connect to RCN API")

        # Save the obtained results and logs
        logger.info("⌛ Checking results existence for experiment")
        code_runner.check_results()
        if code_runner.status_code == TIME_OUT_ERROR:
            logger.error(f"Cannot execute experiment because of Timeout error")
            # Notify the RCN that the experiment execution has not been done
            params = {"experiment_id": experiment_id, "status_code": TIME_OUT_ERROR}
            try:
                rcn_response = requests.post(registration_url, params=params)
                logger.info(
                    f"""RCN API experiment registration request:{params},
                        status_code {rcn_response.status_code},
                        response_request: {rcn_response.text}"""
                )
                return False
            except Exception:
                logger.exception("Cannot connect to RCN API")
        try:
            # Notify the RCN that the experiment execution has been finished
            params = {"experiment_id": experiment_id, "status_code": code_runner.status_code}
            rcn_response = requests.post(registration_url, params=params)
            logger.info(
                f"""RCN API experiment registration request:{params},
                        status_code {rcn_response.status_code},
                        response_request: {rcn_response.text}"""
            )
        except Exception:
            logger.exception("Cannot connect to RCN API")

        # Remove docker image
        logger.info("Removing docker image after execution...")
        removed = code_runner.remove_image()
        if removed:
            logger.info(f"Image removed for experiment_id = {experiment_id}")
        else:
            logger.exception("Exception occured when trying to remove the docker image")

    except MinioConnectionError:
        logger.exception("There has been an error while preparing the experiment path for running the code.")
        # Notify the RCN that the experiment execution has not been done
        params = {"experiment_id": experiment_id, "status_code": TIME_OUT_ERROR}
        rcn_response = requests.post(registration_url, params=params)
        logger.info(
            f"""RCN API experiment registration request:{params},
                    status_code {rcn_response.status_code},
                    response_request: {rcn_response.text}"""
        )
        return False

    except DockerConnectionError:
        logger.exception("There has been an error with the docker connection")
        # Notify the RCN that the experiment execution has not been done
        params = {"experiment_id": experiment_id, "status_code": TIME_OUT_ERROR}
        rcn_response = requests.post(registration_url, params=params)
        logger.info(
            f"""RCN API experiment registration request:{params},
                    status_code {rcn_response.status_code},
                    response_request: {rcn_response.text}"""
        )

        return False


if __name__ == "__main__":
    worker = celery_app.Worker()
    worker.start()



from pathlib import Path
import platform
import os


class LogConfig:
    LOGS_FOLDER_NAME = os.environ.get("LOG_FOLDER_NAME", "logs")
    LOGS_FILE_NAME = os.environ.get("LOG_FILE_NAME", "rai-psm.log")
    LOG_MAX_FILE_SIZE = 1 * 1024 * 1024  # 1MB
    DEBUG = os.environ.get("DEBUG", "True")
    LOG_PATH = Path("./", LOGS_FOLDER_NAME) if platform.uname().system == "Linux" else Path(".", LOGS_FOLDER_NAME)
    if not LOG_PATH.exists():
        LOG_PATH.mkdir(exist_ok=True)
    LOG_DIR = str(LOG_PATH)

    LEVEL = "DEBUG" if DEBUG else "WARN"  # Valid values: DEBUG | INFO | WARN | ERROR | CRITICAL
    CONSOLE_HANDLER = DEBUG
    CONSOLE_HANDLER_LEVEL = "DEBUG"
    FILE_HANDLER = True


class CeleryConfigProcessingScripts:
    MODULE_NAME = "RAI Processing Scripts Manager"
    BROKER_IP = os.environ.get("BROKER_IP", "192.168.65.169")
    BROKER_PORT = os.environ.get("BROKER_PORT", "5672")
    BROKER_CLIENT_USER = os.environ.get("BROKER_CLIENT_USER", "raise")
    BROKER_CLIENT_PWD = os.environ.get("BROKER_CLIENT_PWD", "raise_pwd")
    TOPIC = os.environ.get("PROCESSING_SCRIPTS_TOPIC", "processing-scripts")
    BROKER_URL = f"amqp://{BROKER_CLIENT_USER}:{BROKER_CLIENT_PWD}@{BROKER_IP}:{BROKER_PORT}/{TOPIC}"

    # json serializer is more secure than the default pickle
    CELERY_TASK_SERIALIZER = "json"
    CELERY_ACCEPT_CONTENT = ["json"]
    CELERY_RESULT_SERIALIZER = "json"

    # Use UTC instead of localtime
    CELERY_ENABLE_UTC = True

    # Maximum retries per task
    CELERY_TASK_ANNOTATIONS = {"*": {"max_retries": 3}}

    # A custom property used in tasks.py:run()
    CUSTOM_RETRY_DELAY = 10

    # Kill all long-running tasks with late acknowledgment enabled on connection loss
    CELERYD_CANCEL_LONG_RUNNING_TASKS_ON_CONNECTION_LOSS = True


class MinioConfig:
    # Connection configuration
    MINIO_IP = os.environ.get("MINIO_IP")
    MINIO_USER = os.environ.get("MINIO_USER")
    MINIO_PWD = os.environ.get("MINIO_PWD")
    # Buckets
    DATASETS_BUCKET = os.environ.get("MINIO_BUCKET_DATASETS", "datasets")
    PROCESSING_SCRIPTS_BUCKET = os.environ.get("MINIO_BUCKET_PROCESSING_SCRIPTS", "processing-scripts")
    PROCESSING_SCRIPTS_RESULTS_BUCKET = os.environ.get(
        "MINIO_BUCKET_PROCESSING_SCRIPTS_RESULTS", "processing-scripts-results"
    )



# -*- coding: utf-8 -*-
"""
    RAISE - RAI Processing Scripts Manager

    @author: Mikel Hernández Jiménez - Vicomtech Foundation, Basque Research and Technology Alliance (BRTA)
    @version: 0.1
"""
# Stdlib imports
import os

# import json
import pandas as pd

# Third-party app imports


# Imports from your apps
from code_runner_files.Dataset import Dataset
from code_runner_files.MinioClient import MinioClient
from code_runner_files.custom_exceptions import MinioConnectionError
from code_runner_files.LogClass import LogClass
from code_runner_files.Config import MinioConfig


class LoadData(LogClass):
    """
    Class to store the data corresponding to the requested dataset_id for code running
    """

    def __init__(self, dataset_id: str, data_format: str):
        super().__init__()
        self.dataset_id = dataset_id
        self.data_format = data_format

        self.__data = self.load_data()

    @property
    def data(self):
        return self.__data

    @data.setter
    def data(self, data):
        self.__data = data

    def load_data(self):
        try:
            self.log_info(
                f"Loading dataset from minio, dataset_id = {self.dataset_id}, data_format = {self.data_format}"
            )
            if self.data_format == "csv":
                minio_client = MinioClient()
                minio_client.get_object(
                    bucket_name=MinioConfig.DATASETS_BUCKET,
                    org_file=f"{self.dataset_id}/datafile.csv",
                    dest_file="datafile.csv",
                )
                data = pd.read_csv("datafile.csv")
                os.remove("datafile.csv")

            dataset = Dataset(name=self.dataset_id, data=data)
            return dataset

        except MinioConnectionError:
            self.log_exception("There has been a problem when loading data from Minio")
        except Exception as exc:
            self.log_exception(f"Exception occurred when loading data: {str(exc)}")



import os
import shutil
from importlib import resources
from pathlib import Path


def copy_module_folder(module_name, folder_name, destination):
    """
    Copies the content of a folder within a Python module to a given destination.

    Args:
        module_name (str): The name of the module where the folder resides.
        folder_name (str): The name of the folder to copy.
        destination (str): The destination path where the folder's content should be copied.

    Raises:
        FileNotFoundError: If the specified folder in the module does not exist.
        Exception: If any error occurs during the copy operation.
    """
    try:
        # Resolve the folder path within the module
        with resources.path(module_name, folder_name) as folder_path:
            if not folder_path.is_dir():
                raise FileNotFoundError(f"The folder '{folder_name}' does not exist in module '{module_name}'.")
            
            # Use copytree to copy the entire directory
            shutil.copytree(folder_path, destination, dirs_exist_ok=True)
            
            # print(f"Successfully copied contents of '{folder_name}' from module '{module_name}' to '{destination}'.")
    except Exception as e:
        print(f"An error occurred: {e}")


def copy_module_file(module_name, file_name, destination):
    """
    Copies a single file from a Python module to a specified destination.

    Args:
        module_name (str): The name of the module where the file resides.
        file_name (str): The name of the file to copy.
        destination (str): The full path to the destination (including file name).

    Raises:
        FileNotFoundError: If the specified file in the module does not exist.
        Exception: If any error occurs during the copy operation.
    """
    try:
        # Resolve the file path within the module
        with resources.path(module_name, file_name) as file_path:
            if not file_path.is_file():
                raise FileNotFoundError(f"The file '{file_name}' does not exist in module '{module_name}'.")
            
            # Copy the file to the destination
            shutil.copy(file_path, destination)
            
            # print(f"Successfully copied '{file_name}' from module '{module_name}' to '{destination}'.")
    except Exception as e:
        print(f"An error occurred: {e}")


def clean_folder(folder_path, items_to_keep=None):
    """
    Removes all files and subdirectories from the specified folder except for the given items.

    Args:
        folder_path (str): Path to the folder to clean.
        items_to_keep (list, optional): List of file or subdirectory names to keep (relative to the folder).
                                        If None, the folder will be completely emptied.

    Raises:
        ValueError: If the folder_path does not exist or is not a directory.
    """
    if not os.path.exists(folder_path):
        raise ValueError(f"The folder '{folder_path}' does not exist.")
    if not os.path.isdir(folder_path):
        raise ValueError(f"The path '{folder_path}' is not a directory.")

    # Normalize the items to keep (absolute paths for comparison)
    keep_paths = {os.path.abspath(os.path.join(folder_path, item)) for item in items_to_keep}

    # Iterate over all items in the folder
    for item in os.listdir(folder_path):
        item_path = os.path.join(folder_path, item)

        # If items_to_keep is None, delete everything
        # Otherwise, delete items not in the keep list
        if items_to_keep is None or os.path.abspath(item_path) not in keep_paths:
            if os.path.isfile(item_path) or os.path.islink(item_path):
                os.unlink(item_path)  # Remove file or symlink
            elif os.path.isdir(item_path):
                shutil.rmtree(item_path)  # Remove directory


def find_local_paths(paths):
    """
    Finds the local paths of files relative to the directory containing 'main.py', 'main.js' or 'main.R'.

    Args:
        paths (list of str): List of file paths as strings.

    Returns:
        dict: A dictionary mapping absolute file paths to their relative local paths.

    Raises:
        ValueError: If neither 'main.py' nor 'main.js' nor 'main.R' is found in the provided paths.
    """
    # Convert to Path objects
    path_objects = [Path(p) for p in paths]
    
    # Look for main.*
    main_path = None
    for candidate in ("main.py", "main.js", "main.R"):
        main_path = next((p for p in path_objects if p.name == candidate), None)
        if main_path:
            break
    if main_path is None:
        raise ValueError("No main.* file was found in the provided paths.")
    
    # Determine main.py root directory
    main_root = main_path.parent
    
    # Compute local paths
    local_paths = {p: f"{p.relative_to(main_root)}" for p in path_objects if p != main_path and main_root in p.parents}
    
    return local_paths


# -*- coding: utf-8 -*-
"""
    RAISE - RAI Processing Scripts Manager

    @author: Mikel Hernández Jiménez - Vicomtech Foundation, Basque Research and Technology Alliance (BRTA)
    @version: 0.1
"""
# Stdlib imports
import io

# Third-party app imports
from minio import Minio
from minio.error import S3Error

# Imports from your apps
from code_runner_files.custom_exceptions import MinioConnectionError
from code_runner_files.Config import MinioConfig


class MinioClient:
    def __init__(self):

        self.client = Minio(MinioConfig.MINIO_IP, MinioConfig.MINIO_USER, MinioConfig.MINIO_PWD, secure=False)

    def save_object(self, bucket_name, object_name, data, content_type):
        try:
            data_stream, bytes_len = self.transform_data(data)
            self.client.put_object(
                bucket_name=bucket_name,
                object_name=object_name,
                data=data_stream,
                length=bytes_len,
                content_type=content_type,
            )
        except S3Error:
            raise MinioConnectionError("Error while saving " + object_name + " object in Minio")

    def save_object_from_file(self, bucket_name, object_name, filepath):
        try:
            self.client.fput_object(
                bucket_name=bucket_name,
                object_name=object_name,
                file_path=filepath,
            )
        except S3Error:
            raise MinioConnectionError("Error while saving " + object_name + " object in Minio")

    def get_object(self, bucket_name, org_file, dest_file):
        try:
            self.client.fget_object(bucket_name=bucket_name, object_name=org_file, file_path=dest_file)
        except S3Error:
            raise MinioConnectionError("Error while reading " + org_file + " object from Minio")

    def transform_data(self, data):
        values_as_bytes = data.encode("utf-8")
        values_as_a_stream = io.BytesIO(values_as_bytes)
        return values_as_a_stream, len(values_as_bytes)

    def object_exists(self, bucket_name, object_name):
        objects = self.client.list_objects(bucket_name)
        for ob in objects:
            if object_name in ob.object_name:
                return True
        return False

    def list_folder_files(self, bucket_name, prefix):
        files = []
        objects = self.client.list_objects(bucket_name=bucket_name, prefix=prefix, recursive=True)
        for obj in objects:
            files.append(obj.object_name)
        return files



"""RAISE SDK Remote Execution Validation Operations"""

import os
import sys
path = os.path.abspath(os.path.dirname(__file__))
if path not in sys.path:
    sys.path.append(path)
path = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'app')
if path not in sys.path:
    sys.path.append(path)

from . import app
from .main import processing_script_execution as start

__all__ = [
    "app",
    "start",
]


import os
from PyQt5.QtWidgets import QApplication, QFileDialog, QTreeView, QPushButton, QMessageBox


class FileDialog(QFileDialog):
    def __init__(self, *args):
        super().__init__(*args)
        self.setOption(self.DontUseNativeDialog, True)
        self.setFileMode(self.ExistingFiles)  # Allows file selection
        self.setViewMode(QFileDialog.List)    # Set view mode for better accessibility

        # Find and customize the "Open" button to capture selected items
        btns = self.findChildren(QPushButton)
        self.openBtn = [x for x in btns if 'open' in str(x.text()).lower()][0]
        self.openBtn.clicked.disconnect()
        self.openBtn.clicked.connect(self.openClicked)

        # Access the file tree view to process selections
        self.tree = self.findChild(QTreeView)
        
        # Initialize empty lists for selected files and folders
        self.selectedFiles = []
        self.selectedFolders = []

    def openClicked(self):
        # Process selected files and folders from the tree view
        inds = self.tree.selectionModel().selectedIndexes()
        files = []
        folders = []
        for i in inds:
            if i.column() == 0:  # If the selection is on the first column (file/folder name)
                selected_path = os.path.join(str(self.directory().absolutePath()), str(i.data()))
                if os.path.isfile(selected_path):
                    files.append(selected_path)
                elif os.path.isdir(selected_path):
                    folders.append(selected_path)

        self.selectedFiles = files
        self.selectedFolders = folders
        
        self.hide()

    def filesSelected(self):
        # Return both selected files and folders
        # all_items = self.selectedFiles
        # for root, dirs, files in self.selectedFolders:
        #     all_items += [root + file for file in files]
        return self.selectedFiles + self.selectedFolders


def select_files(title="Select files", filetypes=[("All Files", "*.*")]):
    """
    Opens a file selection dialog and allows the user to select a multiple files.

    Args:
        title (str): The title of the file dialog window.
        filetypes (list): A list of tuples specifying the allowed file types.

    Returns:
        tuple: A tuple containing the paths of the selected files.
    """
    
    app     = QApplication([])  # Create an instance of the application
    options = QFileDialog.Options()
    # Handle the case where filetypes might be a list of a single tuple
    if len(filetypes) == 1 and isinstance(filetypes[0], tuple):
        file_filter = f"{filetypes[0][0]} ({filetypes[0][1]})"
    else:
        # Convert filetypes into the correct string format
        file_filter = ";;".join([f"{name} ({pattern})" for name, pattern in filetypes])
    # Open the file dialog
    file_paths, _ = QFileDialog.getOpenFileNames(None, title, "", file_filter, options=options)

    # Raise an error if no files were selected
    if not file_paths:
        raise FileNotFoundError(f"No files were selected.")

    return file_paths


def show_popup(title="", message=""):
    """
    Show a PyQt5 dialog box asking the user if they want to proceed without Docker.

    Returns:
        bool: True if the user clicks "Yes", False if they click "No".
    """
    app = QApplication([])  # Create a PyQt5 application instance
    msg_box = QMessageBox()
    msg_box.setIcon(QMessageBox.Warning)
    msg_box.setWindowTitle(title)
    msg_box.setText(message)
    msg_box.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
    
    # Execute the dialog and return True for "Yes", False for "No"
    response = msg_box.exec_()
    return response == QMessageBox.Yes


#%%
# ================================== IMPORTS ==================================
import ast
import re
# =============================================================================

#%%
# ================================= PARAMETERS ================================
# Pre‑compile a GUID pattern (matches typical UUID strings)
_GUID_RE = re.compile(
    r"\b[0-9a-fA-F]{8}-"
    r"[0-9a-fA-F]{4}-"
    r"[0-9a-fA-F]{4}-"
    r"[0-9a-fA-F]{4}-"
    r"[0-9a-fA-F]{12}\b"
)
# =============================================================================

#%%
# ================================== CLASSES ==================================
class HardcodedDatasetIDChecker:
    """
    RCP01: Disallow hard-coded dataset UUIDs in code.
    Instead, code should read from the RAISE_DATASET_ID_LIST environment variable.
    """
    name    = "hardcoded-dataset-id"
    version = "0.1.0"
    # Define a unique error code + message template
    _error_code    = "RCP01"
    _error_message = f"{_error_code} Found hard-coded dataset ID; use RAISE_DATASET_ID_LIST env var instead"

    def __init__(self, tree, filename=None):
        self.tree = tree

    def run(self):
        for node in ast.walk(self.tree):
            # look for literal string constants
            if isinstance(node, ast.Constant) and isinstance(node.value, str):
                text = node.value
                if _GUID_RE.search(text):
                    yield (
                        node.lineno,
                        node.col_offset,
                        self._error_message,
                        type(self),
                    )

class PathSeparatorChecker:
    """
    RCP02: Disallow literal Windows path separators in string literals.
    Paths should be built with os.path.join or os.path.sep (or pathlib).
    """
    name    = "path-separator-checker"
    version = "0.1.0"
    # Define a unique error code + message template
    _error_code    = "RCP02"
    _error_message = f"{_error_code} Found literal '\\\\' in path; use os.path.join or os.path.sep"

    def __init__(self, tree, filename=None):
        self.tree = tree

    def run(self):
        for node in ast.walk(self.tree):
            if isinstance(node, ast.Constant) and isinstance(node.value, str):
                text = node.value
                # if there's a backslash (Windows sep) in the literal
                if "\\" in text:
                    yield (
                        node.lineno,
                        node.col_offset,
                        self._error_message,
                        type(self),
                    )
# =============================================================================

#%%
# ================================= FUNCTIONS =================================

# =============================================================================

#%%
# ==================================== MAIN ===================================

# =============================================================================


# -*- coding: utf-8 -*-
"""
    RAISE - RAI Processing Scripts Manager

    @author: Mikel Hernández Jiménez - Vicomtech Foundation, Basque Research and Technology Alliance (BRTA)
    @version: 0.1
"""
# Stdlib imports


# Third-party app imports


# Imports from your apps


class Dataset:
    """
    Abstract class to define type specific Data classes for DataSet

    :param name: Name of the dataset
    """

    def __init__(self, name: str, data):
        super().__init__()
        self._name = name
        self._data = data

    def __getdata__(self):
        return self._data



#%%
# ================================== IMPORTS ==================================
import sys
import subprocess
from pathlib import Path
# =============================================================================

#%%
# ================================= PARAMETERS ================================

# =============================================================================

#%%
# ================================== CLASSES ==================================
class RuffCheckError(Exception):
    """Raised when Ruff reports errors or fails to run."""
    def __init__(self, returncode, output):
        super().__init__(f"Ruff exited with code {returncode}")
        self.returncode = returncode
        self.output = output
# =============================================================================

#%%
# ================================= FUNCTIONS =================================
# Ruff today is a zero‑plugin Python linter: everything you get comes "built‑in",
# and there isn't a public plugin API yet.
# https://docs.astral.sh/ruff/faq/
# https://github.com/astral-sh/ruff/discussions/8409
def run(paths, config_file=None, args=None):
    """
    Run Ruff linter on the given files or directories.

    Uses the `check` subcommand to explicitly invoke lint checks.

    Args:
        paths (List[str] or str): File or list of files/dirs to lint.
        config_file (str): Optional path to a Ruff config.
        args (List[str]): Additional ad-hoc arguments for Ruff.

    Returns:
        str: Combined stdout/stderr from Ruff.

    Raises:
        RuffCheckError: If Ruff exits non-zero.
    """
    if isinstance(paths, (str, Path)):
        paths = [str(paths)]

    # command definition
    cmd = [sys.executable, '-m', 'ruff', 'check']
    if config_file:
        cmd += ['--config', str(config_file)]
    if args:
        cmd += list(args)
    cmd += paths

    # process run
    result = subprocess.run(cmd, capture_output=True, text=True)
    output = (result.stdout or '') + (result.stderr or '')
    if result.returncode != 0:
        raise RuffCheckError(result.returncode, output)
    return output
# =============================================================================

#%%
# ==================================== MAIN ===================================

# =============================================================================


# -*- coding: utf-8 -*-
"""
    RAISE - RAI Processing Scripts Manager

    @author: Mikel Hernández Jiménez - Vicomtech Foundation, Basque Research and Technology Alliance (BRTA)
    @version: 0.1
"""
# Stdlib imports
import abc


# Third-party app imports


# Imports from your apps
from code_runner_files.Dataset import Dataset


class AbstractCodeRunner(metaclass=abc.ABCMeta):
    """
    An abstract class to be inherited by the code to be executed in the RAI node

    :param dataset: Dataset obtained after querying the RAI node. When running this script locally for development, \
        it uses synthetic data. But, run in the RAI node, dataset contains a pointer to real query data in the RAI \
        node.

    """

    def __init__(self) -> None:
        super().__init__()

    @abc.abstractmethod
    def run_code(self):
        """

        Abstract method to be implemented by this API users. This code will be executed on the DataSet.

        :returns: It does not return anything
        :rtype: None
        """
        raise NotImplementedError
